// import { handles } from '@graphpolaris/shared/lib/querybuilder/usecases';
import { useQuerybuilderGraph } from '@graphpolaris/shared/lib/data-access';
import React, { useMemo, useState } from 'react';
import { Handle, Position } from 'reactflow';
import { NodeAttribute, SchemaReactflowEntityNode, handleDataFromReactflowToId, toHandleId } from '../../../model';
import { styleHandleMap } from '../../utils';
import styles from './entitypill.module.scss';
import { PillDropdown } from '../../pilldropdown/pilldropdown';

/**
 * Component to render an entity flow element
 * @param {NodeProps} param0 The data of an entity flow element.
 */
export const EntityFlowElement = React.memo((node: SchemaReactflowEntityNode) => {
  const data = node.data;
  if (!data.leftRelationHandleId) throw new Error('EntityFlowElement: data.leftRelationHandleId is undefined');
  if (!data.rightRelationHandleId) throw new Error('EntityFlowElement: data.rightRelationHandleId is undefined');

  const graph = useQuerybuilderGraph();
  const attributeEdges = useMemo(
    () => graph.edges.filter((edge) => edge.source === node.id && !!edge?.attributes?.sourceHandleData.attributeType),
    [graph]
  );

  const [hovered, setHovered] = useState(false);
  const [handleBeingDragged, setHandleBeingDragged] = useState(-1);

  const onMouseEnter = (event: React.MouseEvent) => {
    if (!hovered) setHovered(true);
  };

  const onMouseLeave = (event: React.MouseEvent) => {
    if (hovered) setHovered(false);
  };

  const onHandleMouseDown = (attribute: NodeAttribute, i: number, event: React.MouseEvent) => {
    setHandleBeingDragged(i);
    window.addEventListener('mouseup', onHandleMouseUp, true);
  };

  const onHandleMouseUp = () => {
    setHandleBeingDragged(-1);
    window.removeEventListener('mouseup', onHandleMouseUp, true);
  };

  const onConnect = (params: any) => {
    console.log('EntityPill onConnect', params);
  };

  return (
    <div className="p-3 bg-transparent" onMouseEnter={onMouseEnter} onMouseLeave={onMouseLeave}>
      <div
        className={`border-l-[3px] border-solid ${
          data.selected ? 'bg-slate-400' : 'bg-entity-50'
        }  border-l-entity-600 font-bold text-xs min-w-[8rem] query_builder-entity`}
      >
        <div>
          <Handle
            id={toHandleId(data.leftRelationHandleId)}
            type="target"
            position={Position.Left}
            className={styles.handle + ' !top-6 !left-2 !bg-entity-800 !rounded-none'}
          />
          <Handle
            id={toHandleId(data.rightRelationHandleId)}
            type="source"
            position={Position.Right}
            className={styles.handle + ' !top-6 !right-2 !bg-entity-800 !rounded-none'}
          />
          <div className="text-center py-1">{data.name}</div>
        </div>
        {data?.attributes && (
          <PillDropdown
            node={node}
            attributes={data.attributes}
            attributeEdges={attributeEdges.map((edge) => edge?.attributes)}
            hovered={hovered}
            handleBeingDraggedIdx={handleBeingDragged}
            onHandleMouseDown={onHandleMouseDown}
          />
        )}
      </div>
    </div>
  );
});

EntityFlowElement.displayName = 'EntityFlowElement';

export default EntityFlowElement;
