import { useMemo } from 'react';
import { NodeAttribute, QueryGraphEdges, SchemaReactflowEntityNode, handleDataFromReactflowToId, toHandleId } from '../../model';
import { sortedIndex } from 'lodash-es';
import { Handle, Position } from 'reactflow';
import styles from './pilldropdown.module.scss';
import { styleHandleMap } from '../utils';

type PillDropdownProps = {
  node: SchemaReactflowEntityNode;
  attributes: NodeAttribute[];
  attributeEdges: (QueryGraphEdges | undefined)[];
  hovered: boolean;
  handleBeingDraggedIdx: number;
  onHandleMouseDown: (attribute: NodeAttribute, i: number, event: React.MouseEvent) => void;
};

export const PillDropdown = (props: PillDropdownProps) => {
  const forceOpen = false;
  const openNumbers: number[] = useMemo(() => {
    if (forceOpen || props.hovered) return props.attributes.map((_, i) => i);

    //collect connected nodes
    const connected = props.attributes
      .map((attribute) => {
        const nodeID = handleDataFromReactflowToId(props.node, attribute);
        return props.attributeEdges.some((edge) => edge?.sourceHandleData && toHandleId(edge?.sourceHandleData) === nodeID);
      })
      .map((connected, i) => (connected ? i : -1))
      .filter((i) => i !== -1);

    // add to it the node being dragged
    if (props.handleBeingDraggedIdx !== -1 && !connected.includes(props.handleBeingDraggedIdx)) connected.push(props.handleBeingDraggedIdx);
    return connected.sort();
  }, [props]);

  return (
    <>
      <div className={'' + (openNumbers.length > 0 ? 'animate-openmenu ' : 'animate-closemenu ')}>
        {openNumbers.map((i) => {
          const attribute = props.attributes[i];
          if (attribute.handleData.attributeName === undefined) {
            throw new Error('attribute.handleData.attributeName is undefined');
          }

          return (
            <div
              className="px-5 py-1 font-semibold"
              key={(attribute.handleData.attributeName || '') + i}
              onMouseDown={(event: React.MouseEvent) => {
                props.onHandleMouseDown(attribute, i, event);
              }}
            >
              {attribute.handleData.attributeName}
              <Handle
                id={handleDataFromReactflowToId(props.node, attribute)}
                type="source"
                position={Position.Right}
                className={
                  styles.handle +
                  ' ' +
                  (attribute.handleData.attributeType ? styleHandleMap[attribute.handleData.attributeType] : '') +
                  ' !top-auto !-translate-y-[145%] !right-[1.2rem]'
                }
              ></Handle>
            </div>
          );
        })}
      </div>
    </>
  );
};
